<?php

/* $Id: media_mover.inc,v 1.9 2007/02/02 21:55:09 arthuregg Exp $ */

/**
 * This file contains all the media mover functions for the media_mover hook 
 * which are seperate from the internal workings of the media mover module
 * 
 * all functions which are core to media mover which are not called from the 
 * media mover hook should be in the media_mover.module file
 */


/**
 * Implementation of media_mover hook
 */
function media_mover_media_mover($op, $action_id = null, $configuration = null, &$file = array() ) {

  switch ($op) {
    
    // give your driver a distinct name
    case 'name':
      return "Media Mover module";
    break;
    
    // create edit configuration option set
    // $action is used to ensure namespacing is kept consistent, form 
    // elements must be named $form[{$action_id}:your_variable_name]
    // $configuration is an array, settings stored on a per config basis
    case 'config':
      switch($action_id){
      	// setup the bypass options
      	case "harvest:1": //bypass
      	  return array("#title" => "Bypass");
      	break;
      	case "process:1": //bypass
      	  return array("#title" => "Bypass");
      	break;
      	case "storage:1": //bypass
      	  return array("#title" => "Bypass");
      	break;
        case "complete:1": //bypass
      	  return array("#title" => "Bypass");
      	break;
       	// here are internal media mover options      	
      	case "harvest:2": //select_drupal_uploaded_files
          return _media_mover_admin_harvest_config($action_id, $configuration);
        break;
        case "complete:3": //set node status
          return _media_mover_admin_complete_config($action_id, $configuration);
        break;
    }
    break;
    
    // set standard configuration for this module
    // this is the global config. Individual config items
    // should be in "new config"
    // displayed on admin/settings/media_mover
    // returns a form array
    case 'admin':
      return _media_mover_admin();
    break;
    
    // defines what type of driver this module is
    // array of harvest, process, storage, complete
    // a module may operate more than one verb
    case 'verbs':    
      return array("harvest", "process", "storage", "complete");    
    break;   
  
    // defines the actions that this module does
    // these are numbered for easy identification of what
    // action is being called
    case 'actions':
      return array( 1 => "Bypass this operation", 
                    2 => "select drupal uploaded files", 
                    3 => "set node status"
                  );
    break;
  
    // defines directories (under master media_mover directory) this module uses 
    // array of directories, will be created under the default
    // media_mover directory, set in admin
    case 'directories':
    break;
    
    // allows for module to add additional data in a different db 
    // add file to db action. called from media_mover's mm_files_db_add
    // by default, medial_mover's files table is used. $file is available
    case 'add':
    break;
    
    // allows for module to fetch additional data 
    // added to $file to db action. called from media_mover's mm_files_db_fetch
    // by default, medial_mover's files table is used. $file is available
    case 'fetch':
    break;
    
    // allows for module to update additional critera 
    // add file to db action. called from media_mover's mm_files_db_update
    // by default, medial_mover's files table is used. $file is available
    case 'update':
    break;
    
    // functions called on harvest op
    // returns an array of $files
    case 'harvest':
      return _mm_harvest($action_id, $configuration); 
    break;
    
    // functions called on process op
    case 'process':  
    break;
    
    // functions called on storage op
    case 'storage':  
    break;
    
    // functions called on completion
    case 'complete':    
      return _mm_complete($configuration, $file);
    break;
    
    // custom theme a file assoicated with this module
    case 'theme':
    break;
    
    default:
      return;
    break;
  }
  return;
}


/* ************************************************ */
/* Media Mover: Media Mover hook  functions */
/* ************************************************ */

/**
 * Get list of files that match assumed critera
 * 
 * returns a joined array from node and file tables 
 */
function _mm_harvest($action, $configuration){
  if ($node_types = $configuration['mm_node_types'] ){
      foreach ($node_types as $type) {
        if ($type != "0" ){
            $node_type_list[] = "'" . $type . "'";
        }
      }
    
    // make sure $node_type_list is populated
    if($node_type_list) { 
      $node_type_list = implode(" ," , $node_type_list) ;
    }
    else { return; }
    
    // get all potentially harvestable files
    // @ TODO is this a security hole? 
    $query = "SELECT * FROM {files} f " . 
           "LEFT JOIN {node} n ON f.nid = n.nid ".
           "WHERE ((n.type IN ($node_type_list)) AND (n.status != 1) AND (n.moderate != 1))";
    $results = db_query($query);

    // query db to get list of already harvested files
    $query = "SELECT * FROM {media_mover_files} WHERE harvest_module = 'media_mover' AND cid = %d";
    $previously_harvested = db_query($query, $configuration['cid']);
    while($result = db_fetch_array($previously_harvested)){
      $harvested[$result['nid']] = $result;
    }
    while ($result = db_fetch_array($results) ) {
      // check harvested file against previously harvested
      if (!($harvested[$result['nid']])){
        // add an absolute path onto the filepath
        $result['filepath'] = $_SERVER['DOCUMENT_ROOT'] . base_path() . $result['filepath'];
        $files[] = $result;
      }    
    }
  }
  return $files;
}

/**
 * Get list of files that match assumed critera
 * 
 * returns a joined array from node and file tables 
 */
function _mm_complete( $configuration, $file){
  if ($configuration['complete_conditions']['published'] != '0'){ $options[] = "status = 1 ";}
  if ($configuration['complete_conditions']['in moderation que'] != '0'){ $options[] = "moderate = 1 ";}
  if ($configuration['complete_conditions']['promoted to front page'] != '0'){ $options[] = "promote = 1 ";}
  $options = implode(", ", $options);
  db_query("UPDATE node SET $options WHERE nid = %d", $file->nid);
}

/**
 * Admin form for media mover 
 * @ TODO move the types configuration over to the individual configs
 */
function _media_mover_admin(){

  $form['mm_config']['mm_file_dir'] = array(
   '#type' => 'textfield',
   '#title' => t('Converted files storage'),
   '#description' => t('Sets the directory where media_mover files are stored.'),
   '#default_value' =>  variable_get('mm_file_dir', file_create_path("media_mover")),
  );
  
  return $form;
}


/**
 * Config config form for media mover 
 * creates the options for each configuration instance
 * 
 * NOTE: name form files like: ["{$action}:db_options"] 
 * so that they are processed correctly on the add configuration 
 * screen
 */
function _media_mover_admin_harvest_config($action_id, $configuration) {
  
  $javascript = array("onclick" => "javascript: alert(this.value)");
  
  $form["mm_harvest_conf"] = array(
    '#type' => 'fieldset',
    '#title' => t('Harvest Configuration'),
    '#collapsible' => true,
    '#collapsed' => true,
    '#attributes' => array("id" => "media_mover:2"),  // modulename:action, this allows the javascript auto hide and show on the add config page
  );


  foreach (node_get_types() as $type){
    $node_types[$type->type] = $type->name;
  }

  $form['mm_harvest_conf']["{$action_id}:mm_node_types"] = array(
   '#type' => 'checkboxes',
   '#title' => t('Select files from these content types'),
   '#description' => t('Select content types you wish to search for video files to convert.'),
   '#options' => $node_types,
   '#default_value' => $configuration['mm_node_types'],
  );

  $options = array("published" => t("Published"), "in moderation que" => t("In moderation que"), "promoted to front page" => t("Promoted to front page"));
  $form['mm_harvest_conf']["{$action_id}:db_options"] = array(
    '#type' => "checkboxes",
    '#options' => $options,
    '#default_value' => $configuration['db_options'],
    '#title' => t('Conditions to harvest from drupal'),
    '#description' => t('Check boxes to select. All of these conditions must be true to harvest a file.'),
    );
  return $form;
}


/**
 * Config config form for media mover 
 * creates the options for each configuration instance
 * 
 * NOTE: name form files like: ["{$action}:db_options"] 
 * so that they are processed correctly on the add configuration 
 * screen
 */
function _media_mover_admin_complete_config($action, $configuration) {
  $form["complete"] = array(
    '#type' => 'fieldset',
    '#title' => t('Completion Configuration'),
    '#collapsible' => true,
    '#collapsed' => true,
    '#attributes' => array("id" => "media_mover:3"), //module name:id
  );

  $options = array("published" => t("Published"), "in moderation que" => t("In moderation que"), "promoted to front page" => t("Promoted to front page"));
  $form['complete']["{$action}:complete_conditions"] = array(
    '#title' => t('Completion condition'),
    '#type' => "checkboxes",
    '#options' => $options,
    '#default_value' => $configuration['complete_conditions'],
    '#description' => t('Set the status of the node post completion'),
    );
  return $form;
}

